/**
 *############################################################################
 * A component of the Greenstone Librarian Interface, part of the Greenstone
 * digital library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: Michael Dewsnip, NZDL Project, University of Waikato, NZ
 *
 * Copyright (C) 2004 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *############################################################################
 */

package org.greenstone.gatherer.greenstone;


import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.DebugStream;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;


public class LocalLibraryServer
{
    static final private String ADD_COMMAND = "?a=config&cmd=add-collection&c=";
    static final private String RELEASE_COMMAND = "?a=config&cmd=release-collection&c=";
    static final private String QUIT_COMMAND = "?a=config&cmd=kill";

    static private GSDLSiteConfig gsdlsite_cfg_file = null;
    static private File local_library_server_file = null;

    static private boolean running = false;

    static public void addCollection(String collection_name)
    {
	config(ADD_COMMAND + collection_name);
    }


    // Used to send messages to the local library
    static private void config(String command)
    {
	if (Configuration.library_url == null) {
	    System.err.println("Error: Trying to configure local library with null Configuration.library_url!");
	    return;
	}

	try {
	    URL url = new URL(Configuration.library_url.toString() + command);
	    HttpURLConnection library_connection = (HttpURLConnection) url.openConnection();

	    // It's very important that we read the output of the command
	    // This ensures that the command has actually finished
	    // (The response code is returned immediately)
	    InputStream library_is = library_connection.getInputStream();
	    BufferedReader library_in = new BufferedReader(new InputStreamReader(library_is, "UTF-8"));
	    String library_output_line = library_in.readLine();
	    while (library_output_line != null) {
		DebugStream.println("Local library server output: " + library_output_line);
		library_output_line = library_in.readLine();
	    }
	    library_in.close();

	    int response_code = library_connection.getResponseCode();
	    if (response_code >= HttpURLConnection.HTTP_OK && response_code < HttpURLConnection.HTTP_MULT_CHOICE) {
		DebugStream.println("200 - Complete.");
	    }
	    else {
		DebugStream.println("404 - Failed.");
	    }
	}
	catch (Exception exception) {
	    DebugStream.printStackTrace(exception);
	}
    }


    static public boolean isRunning()
    {
	if (!running) return false;
	gsdlsite_cfg_file.load();
	if (gsdlsite_cfg_file.getURL() == null)  return false;
	return true;
    }


    static public void releaseCollection(String collection_name)
    {
	config(RELEASE_COMMAND + collection_name);
    }


    static public void start(String gsdl_path, String local_library_server_file_path)
    {
	// Check the local library server.exe file exists
	local_library_server_file = new File(local_library_server_file_path);
	if (!local_library_server_file.exists()) {
	    DebugStream.println("No local library at given file path.");

	    local_library_server_file = new File(gsdl_path + "server.exe");
	    if (!local_library_server_file.exists()) {
		DebugStream.println("No local library at all.");
		return;
	    }
	}

	// Check if the server is already running
	gsdlsite_cfg_file = new GSDLSiteConfig(local_library_server_file);
	String url = gsdlsite_cfg_file.getURL();
	if (url != null) {
	    // If it is already running then set the Greenstone web server address and we're done
	    try {
		Configuration.library_url = new URL(url);
		running = true;
		return;
	    }
	    catch (MalformedURLException exception) {
		DebugStream.printStackTrace(exception);
	    }
	}

	// Configure the server for immediate entry
	gsdlsite_cfg_file.set();

	// Spawn local library server process
	String local_library_server_command = local_library_server_file.getAbsolutePath() + " " + gsdlsite_cfg_file.getSiteConfigFilename();
	Gatherer.spawnApplication(local_library_server_command);

	// Wait until program has started, by reloading and checking the URL field
	gsdlsite_cfg_file.load();
	int attempt_count = 0;
	while (gsdlsite_cfg_file.getURL() == null) {
	    new OneSecondWait();  // Wait one second (give or take)
	    gsdlsite_cfg_file.load();
	    attempt_count++;

	    // After waiting a minute ask the user whether they want to wait another minute
	    if (attempt_count == 60) {
		int try_again = JOptionPane.showConfirmDialog(Gatherer.g_man, Dictionary.get("Server.QuitTimeOut"), Dictionary.get("General.Warning"), JOptionPane.YES_NO_OPTION);
		if (try_again == JOptionPane.NO_OPTION) {
		    return;
		}
		attempt_count = 0;
	    }
	}

	// Ta-da. Now the url should be available
	try {
	    Configuration.library_url = new URL(gsdlsite_cfg_file.getURL());
	}
	catch (MalformedURLException exception) {
	    DebugStream.printStackTrace(exception);
	}

	// A quick test involves opening a connection to get the home page for this collection
	try {
	    DebugStream.println("Try connecting to server on config url: '" + Configuration.library_url+ "'");
	    URLConnection connection = Configuration.library_url.openConnection();
	    connection.getContent();
	}
	catch (IOException bad_url_connection) {
	    try {
		// If this fails then we try changing the url to be localhost
		Configuration.library_url = new URL(gsdlsite_cfg_file.getLocalHostURL());
		DebugStream.println("Try connecting to server on local host: '" + Configuration.library_url + "'");
		URLConnection connection = Configuration.library_url.openConnection();
		connection.getContent();
	    }
	    catch (IOException worse_url_connection) {
		DebugStream.println("Can't connect to server on either address.");
		Configuration.library_url = null;
		return;
	    }
	}

	running = true;
    }


    static public void stop()
    {
	if (running == false) {
	    return;
	}

	// Send the command for it to exit.
	config(QUIT_COMMAND);

	// Wait until program has stopped, by reloading and checking the URL field
	gsdlsite_cfg_file.load();
	int attempt_count = 0;
	while (gsdlsite_cfg_file.getURL() != null) {
	    new OneSecondWait();  // Wait one second (give or take)
	    gsdlsite_cfg_file.load();
	    attempt_count++;

	    // After waiting a minute ask the user whether they want to wait another minute
	    if (attempt_count == 60) {
		int try_again = JOptionPane.showConfirmDialog(Gatherer.g_man, Dictionary.get("Server.QuitTimeOut"), Dictionary.get("General.Warning"), JOptionPane.YES_NO_OPTION);
		if (try_again == JOptionPane.NO_OPTION) {
		    return;
		}
		attempt_count = 0;
	    }
	}

	// Restore the gsdlsite_cfg.
	gsdlsite_cfg_file.restore();

	// If the local server is still running then our changed values will get overwritten.
	if (gsdlsite_cfg_file.getURL() != null) {
	    JOptionPane.showMessageDialog(Gatherer.g_man, Dictionary.get("Server.QuitManual"), Dictionary.get("General.Error"), JOptionPane.ERROR_MESSAGE);
	}

	running = false;
    }
    
    static public void checkServerRunning() {
	if (!running) return; // don't worry about it if its not supposed to be running 
	gsdlsite_cfg_file.load();
	if (gsdlsite_cfg_file.getURL() == null) {
	    // need to restart the server again
	    gsdlsite_cfg_file.set();

	    // Spawn local library server process
	    String local_library_server_command = local_library_server_file.getAbsolutePath() + " " + gsdlsite_cfg_file.getSiteConfigFilename();
	    Gatherer.spawnApplication(local_library_server_command);

	}
    }
    static private class OneSecondWait
    {
	public OneSecondWait()
	{
	    synchronized(this) {
		try {
		    wait(1000);
		}
		catch (InterruptedException exception) {
		}
	    }
	}
    }


    static public class GSDLSiteConfig
	extends LinkedHashMap {
	private File gsdlsite_cfg;
	private File glisite_cfg;
	private String autoenter_initial;
	private String start_browser_initial;
    
	static final private String AUTOENTER = "autoenter";
	static final private String COLON = ":";
	static final private String ENTERLIB = "enterlib";
	static final private String FALSE = "0";
	static final private String GLISITE_CFG = "glisite.cfg";
	static final private String GSDL = "gsdl";
	static final private String GSDLSITE_CFG = "gsdlsite.cfg";
	static final private String LOCAL_HOST = "http://localhost"; 
	static final private String PORTNUMBER = "portnumber";
	static final private String SEPARATOR = "/";
	static final private String SPECIFIC_CONFIG = "--config=";
	static final private String STARTBROWSER = "start_browser";
	static final private String TRUE = "1";
	static final private String URL = "url";

	public GSDLSiteConfig(File server_exe) {
	    debug("New GSDLSiteConfig for: " + server_exe.getAbsolutePath());
	    gsdlsite_cfg = new File(server_exe.getParentFile(), GSDLSITE_CFG);
	    glisite_cfg = new File(server_exe.getParentFile(), GLISITE_CFG);
	    autoenter_initial = null;
	    start_browser_initial = null;
	    if(gsdlsite_cfg.exists()) {
		debug("Load: " + gsdlsite_cfg.getAbsolutePath());
		clear();
		try {
		    BufferedReader in = new BufferedReader(new FileReader(gsdlsite_cfg));
		    String line = null;
		    while((line = in.readLine()) != null) {
			String key = null;
			String value = null;
			int index = -1;
			if((index = line.indexOf("=")) != -1 && line.length() >= index + 1) {
			    key = line.substring(0, index);
			    value = line.substring(index + 1);
			}
			else {
			    key = line;
			}
			put(key, value);
		    }
		    in.close();
		}
		catch (Exception error) {
		    error.printStackTrace();
		}
	    }
	    else {
		debug("No GSDLsite.cfg file can be found!");
	    }
	}

	public boolean exists() {
	    return gsdlsite_cfg.exists();
	}

	public String getLocalHostURL() {
	    StringBuffer url = new StringBuffer(LOCAL_HOST);
	    url.append(COLON);
	    url.append((String)get(PORTNUMBER));
	    String enterlib = (String)get(ENTERLIB);
	    if(enterlib == null || enterlib.length() == 0) {
		// Use the default /gsdl and hope for the best.
		url.append(SEPARATOR);
		url.append(GSDL);
	    }
	    else {
		if(!enterlib.startsWith(SEPARATOR)) {
		    url.append(SEPARATOR);
		}
		url.append(enterlib);
	    }
	    enterlib = null;
	    debug("Found Local Library Address: " + url.toString());
	    return url.toString();
	}

	public String getSiteConfigFilename() {
	    return SPECIFIC_CONFIG + glisite_cfg.getAbsolutePath();
	}

	public String getURL() {
	    // URL is made from url and portnumber
	    String url = (String) get(URL);
	    if(url != null) {
		StringBuffer temp = new StringBuffer(url);
		temp.append(COLON);
		temp.append((String)get(PORTNUMBER));
		String enterlib = (String)get(ENTERLIB);
		if(enterlib == null || enterlib.length() == 0) {
		    // Use the default /gsdl and hope for the best.
		    temp.append(SEPARATOR);
		    temp.append(GSDL);
		}
		else {
		    if(!enterlib.startsWith(SEPARATOR)) {
			temp.append(SEPARATOR);
		    }
		    temp.append(enterlib);
		}
		enterlib = null;
		url = temp.toString();
	    }
	    debug("Found Local Library Address: " + url);
	    return url;
	}

	public void load() {
	    if(glisite_cfg.exists()) {
		debug("Load: " + glisite_cfg.getAbsolutePath());
		clear();
		try {
		    BufferedReader in = new BufferedReader(new FileReader(glisite_cfg));
		    String line = null;
		    while((line = in.readLine()) != null) {
			String key = null;
			String value = null;
			int index = -1;
			if((index = line.indexOf("=")) != -1 && line.length() >= index + 1) {
			    key = line.substring(0, index);
			    value = line.substring(index + 1);
			}
			else {
			    key = line;
			}
			put(key, value);
		    }
		    in.close();
		}
		catch (Exception error) {
		    error.printStackTrace();
		}
	    }
	    else {
		debug("No GSDLsite.cfg file can be found!");
	    }
	}

	/** Restore the autoenter value to its initial value, and remove url if present. */
	public void restore() {
	    if(glisite_cfg != null) {
		// Delete the file
		glisite_cfg.delete();
	    }
	    else {
		debug("Restore Initial Settings");
		put(AUTOENTER, autoenter_initial);
		put(STARTBROWSER, start_browser_initial);
		remove(URL);
		save();
	    }
	}

	public void set() {
	    debug("Set Session Settings");
	    if(autoenter_initial == null) {
		autoenter_initial = (String) get(AUTOENTER);
		debug("Remember autoenter was: " + autoenter_initial);
	    }
	    put(AUTOENTER, TRUE);
	    if(start_browser_initial == null) {
		start_browser_initial = (String) get(STARTBROWSER);
		debug("Remember start_browser was: " + start_browser_initial);
	    }
	    put(STARTBROWSER, FALSE);
	    save();
	}

	private void debug(String message) {
	    ///ystem.err.println(message);
	}

	private void save() {
	    //debug("Save: " + gsdlsite_cfg.getAbsolutePath());
	    debug("Save: " + glisite_cfg.getAbsolutePath());
	    try {
		//BufferedWriter out = new BufferedWriter(new FileWriter(gsdlsite_cfg, false));
		BufferedWriter out = new BufferedWriter(new FileWriter(glisite_cfg, false));
		for(Iterator keys = keySet().iterator(); keys.hasNext(); ) {
		    String key = (String) keys.next();
		    String value = (String) get(key);
		    out.write(key, 0, key.length());
		    if(value != null) {
			out.write('=');
			out.write(value, 0, value.length());
		    }
		    out.newLine();
		}
		out.flush();
		out.close();
	    }
	    catch (Exception error) {
		error.printStackTrace();
	    }
	}
    }
}
